#import multiprocessing
from threading import Thread, Lock
from utils import transport
from utils.ml import AnomalyDetection
import time
import monitor
import sys
import os
"""
	This class is intended to collect data given a configuration

"""
class Top(Thread):
	def __init__(self,_config,lock):
		Thread.__init__(self)
		self.lock = lock
		self.reader_class	= _config['store']['class']['read']
		self.write_class	= _config['store']['class']['write']
		self.rw_args		= _config['store']['args']
		self.factory 		= transport.DataSourceFactory()
		
		self.name = 'Zulu-Top'
		self.quit = False

		
		className = ''.join(['monitor.',_config['monitor']['processes']['class'],'()'])
		self.handler = eval(className)
		self.config = _config['monitor']['processes']['config']
	def stop(self):
		self.quit = True
	def run(self):
		while self.quit == False:
			for label in self.config :
				self.lock.acquire()
				gwriter = self.factory.instance(type=self.write_class,args=self.rw_args)
				apps = self.config[label]
				self.handler.init(apps)	
				r = self.handler.composite()
				gwriter.write(label=label,row=r)
				time.sleep(5)
				self.lock.release()
			if 'MONITOR_CONFIG_PATH' in os.environ:
				#
				# This suggests we are in development mode
				#
				break
			ELLAPSED_TIME = 60*30
			time.sleep(ELLAPSED_TIME)
		print "Exiting ",self.name
				
class Learner(Thread) :
	
	"""
		This function expects paltform config (store,learner)
		It will leverage store and learner in order to operate
	"""
	def __init__(self,config,lock):
		Thread.__init__(self)
		self.name='Zulu-Learner'
		self.lock = lock
		self.reader_class	= config['store']['class']['read']
		self.write_class	= config['store']['class']['write']
		self.rw_args	= config['store']['args']
		self.features = config['learner']['anomalies']['features']
		self.yo	= config['learner']['anomalies']['label']
		self.apps = config['learner']['anomalies']['apps']
		self.factory 		= transport.DataSourceFactory()
	"""
		This function will initiate learning every (x-hour)
		If there is nothing to learn the app will simply go to sleep
	"""
	def run(self):
		reader = self.factory.instance(type=self.reader_class,args=self.rw_args)
		data = reader.read()
		#
		# This is the motherload of innefficiency ...
		#
		while True:
			r = {}
			for key in data :
				logs = data[key]
				
				for app in self.apps:
					
					handler = AnomalyDetection()
					value = handler.learn(logs,'label',app,self.features,self.yo)
					
					if value is not None:
						print value
						if key not in r:
							r[key] = {}
						r[key][app] = value
			#
			# At this point we've already learnt every thing we need to learn
			#
			
			if r.keys() :
				
				self.lock.acquire()
				writer = self.factory.instance(type=self.write_class,args=self.rw_args)
				writer.write(label='learn',row=r)
				self.lock.release()

			if 'MONITOR_CONFIG_PATH' in os.environ:
				#
				# This suggests we are in development mode
				#
				break

			TIME_ELLAPSED = 60*120	#-- Every 2 hours
			time.sleep(TIME_ELLAPSED)
			



